use clap::{Command, ArgMatches};

#[derive(thiserror::Error, Debug)]
#[error("The requested operation is unsupported or not yet implemented")]
pub struct Unsupported;

pub trait Coin {
    /// Return the name for the coin, in lowercase.
    fn coin_name(&self) -> &'static str;

    /// Return a description of the coin.
    fn coin_description(&self) -> &'static str;

    /// Given a Command, add any arguments relevant to the Transfer operation.
    ///
    /// All commands by default have the following values:
    /// * `amount`: The amount of the currency to transfer. May be a float.
    /// * `to`: The account to transfer the currency to.
    /// * `from_account`: The non-default derivation account to send currency from.
    fn build_transfer_command(&self, _command: Command) -> Result<Command, Unsupported> {
        Err(Unsupported)
    }

    /// Given a Command, add any arguments relevant to the Stake operation.
    fn build_stake_command(&self, _command: Command) -> Result<Command, Unsupported> {
        Err(Unsupported)
    }

    /// Perform a Transfer
    ///
    /// TODO: This should later be split into "make a transfer",
    /// "verify a transfer", "sign a transfer", and "broadcast the transfer".
    #[allow(unused_variables)]
    fn run_transfer_command(&self, matches: &ArgMatches) -> Result<(), Box<dyn std::error::Error>> {
        Err(Box::new(Unsupported))
    }
}
